# file: SudokuCGI.pm
#
# (c) Copyright J.Fossy Weinzinger 2005
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Fossy: 2005.11.03
# Fossy: 2005.11.14
#

use warnings;
use strict;

use CGI qw/:standard *table *Tr *p/;
use Storable qw(lock_store lock_retrieve);


#----------------------------------------------------------------------------
use constant STATFILE => '.sudoku.status';

my %statistic;

  
#----------------------------------------------------------------------------
sub SudokuTableSmall($$$)
{
  my ($elementFunction, $bigrow, $bigcol) = @_;
  my @result;
  
  push @result, start_table({ -class => 'small' });
  
  for my $row (0..2)
    {
      push @result, start_Tr;
      
      for my $col (0..2)
        {
          my ($value, $class) = $elementFunction->(3*$bigrow + $row, 3*$bigcol + $col);
          
          push @result, td( { -class => $class }, $value );
        }
        
      push @result, end_Tr, "\n";
    }
    
  push @result, end_table, "\n";
  
  return @result;
}


#----------------------------------------------------------------------------
sub SudokuTable($)
{
  my $elementFunction = shift;
  my @result;

  push @result, start_table({ -class => 'big' });
  
  for my $bigrow (0..2)
    {
      push @result, start_Tr;
      
      for my $bigcol (0..2)
        {
          push @result, td({ -class => 'sq' }, SudokuTableSmall($elementFunction, $bigrow, $bigcol));
        }
        
      push @result, end_Tr, "\n";
    }
    
  push @result, end_table, "\n";
  
  return @result;
}


#----------------------------------------------------------------------------
sub GetStatistic()
{
  eval
    {
      %statistic = %{ lock_retrieve(STATFILE) };
    };
        
}


#----------------------------------------------------------------------------
sub WriteStatistic($)
{
  #
  # the trick is there are two "verbs" 'solved' and 'generated'
  # which are used as key in the %statisic hash and to generate
  # the sentence - this is wy it's called "verb"
  #
  my $verb = shift;
  my @keys = keys %{ $statistic{$verb} };
  
  if ( !$statistic{'since'} )
    {
      $statistic{'since'} = time();
      
      lock_store(\%statistic, STATFILE);
    }
  
  if ( @keys )
    {
      my $lastVal = $statistic{$verb}->{$keys[-1]};
      
      map { $_ = $statistic{$verb}->{$_} . ' ' . $_ } @keys;
      
      print start_p, ucfirst($verb), ' ';
      
      for my $i (0..$#keys-2)
        {
          print $keys[$i], ', ';
        }
        
      print $keys[-2], ' and ' if @keys > 1;
      print $keys[-1];
           
      my $since = localtime($statistic{'since'});

      printf " puzzle%s - since %s", ($lastVal > 1 ? 's' : ''), $since;
      print end_p, "\n";
    }
}


#----------------------------------------------------------------------------
sub UpdateStatistic($$)
{
  my ($verb, $class) = @_;
  
  $statistic{$verb}->{$class}++;
    
  lock_store(\%statistic, STATFILE);
}

  
#----------------------------------------------------------------------------
1;