#!/usr/bin/perl
#
# file: solve.cgi 
#
# (c) Copyright J.Fossy Weinzinger 2005
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Fossy: 2005.11.03
# Fossy: 2005.11.14
# Fossy: 2005.11.19
#

use warnings;
use strict;

use Time::HiRes qw/gettimeofday tv_interval/;
use Sudoku;     # Sudoku.pm in the same directory
use SudokuCGI;  # SudokuCGI.pm in the same directory


#----------------------------------------------------------------------------
sub ReadSquare($)
{
  my $square = shift;
  my $error  = 0;
  
  @$square = ();
  
  for my $row (1..9)
    {
      for my $col (1..9)
        {
          my $val1   = param($row . $col);
          my ($val2) = $val1 =~ /(^[1-9]$)/;
          
          push @$square, $val2;
          
          $error++ if defined($val1) && $val1 ne $val2;
        }
    }
    
  return $error;
}


#----------------------------------------------------------------------------
sub ResultCell($$$$)
{
  my ( $index,
       $result,
       $lastres,
       $square
     ) = @_;
     
  my $val  = $result ->[$index];
  my $lval = $lastres->[$index];
  my $sval = $square ->[$index];
  
  return ( p( { -class => 'cell' }, $val),
           $sval && $val == $sval
             ? 'fix'
             : $val == $lval
                  ? 'normal'
                  : 'alter'
         );
}


#----------------------------------------------------------------------------
sub WriteResults($$$@)
{
  my $maxres  = shift;
  my $time    = shift;
  my $square  = shift;
  my @results = @_;
  my $numres  = @results;
  
  if ( ! $numres )
    {
      print p('sorry - no results');
      return;  #============================================================>
    }
  
  print p(sprintf "found %d result%s in %.3f seconds%s", 
                  $numres, 
                  $numres == 1 ? '' : 's',
                  $time,
                  $numres == $maxres ? ' (maybe there are more)' : ''
         );
         
  print p('class is ', em(Sudoku::Class), sprintf(" [%d]", Sudoku::Measure));
  
  print start_table, start_Tr;
  print td(p({ -class => 'fix' },   'given'))                    if @results > 0;
  print td(p({ -class => 'alter' }, 'altered from last result')) if @results > 1;
  print end_Tr, end_table, p;
  
  for ( my ($idx, $lastres) = (0, $results[0]);
        $idx < @_;
        $lastres = $results[$idx++]
      )
    {
      print SudokuTable(sub { ResultCell(9*$_[0] + $_[1], $results[$idx], $lastres, $square); }),
            br, "\n";
    }
    
  print p('maybe there are more results') if $numres == $maxres;
}


#----------------------------------------------------------------------------

print header,
      start_html( -title => 'Sudoku Solver',
                  -style => {-src => 'sudoku.css'},
                  -head  => Link({ -rel => 'shortcut icon', -href => 'sudoku.ico' })
                ),
      "\n",
      p({ -class => 'right' }, a({ -href => '.' }, '&lt;back')),
      h1('Enter a sudoku puzzle'),
      start_form,
      "\n",
      SudokuTable(sub { (p({ -class => 'cell' }, textfield( -name => ($_[0] + 1) . ($_[1] + 1), -size => 1, -maxlength => 1 )), 'normal') } ),
      p( 'accept a maximum of ',
         textfield( -name => 'maxres', -value => 15, -size => 3, -align => 'right' ),
         'results'
       ),
      submit('submit'),
      '&nbsp;',
      reset('clear'),
      end_form, "\n";

if ( param() )
  {
    my $maxres  = param('maxres');
    my @square;
    my $errors = ReadSquare(\@square);
     
    $maxres = 15 if !defined($maxres) || $maxres <= 0;
    
    my $t0      = [ gettimeofday ];
    my @results = Sudoku::Solve( $maxres, @square);
    my $time    = tv_interval($t0);    

    GetStatistic;
    UpdateStatistic('solved', Sudoku::Class);
    WriteStatistic('solved');
    print hr, "\n";
    
    if ( $errors )
      {
        #
        # some errors 
        #
        print p('Please enter only digits from 1 .. 9 !'), "\n";
      }
    
    WriteResults($maxres, $time, \@square, @results);
  }
else
  {
    GetStatistic;
    WriteStatistic('solved');
    print hr, "\n";
  }

print p({ -class => 'right' }, a({ -href => '.' }, '&lt;back')), "\n";
